// ======================================================================
//
// gl_wu2.h
// jeff grills
//
// copyright 1998 origin systems inc
//
// ======================================================================

#ifndef GL_WU2_H
#define GL_WU2_H

// ======================================================================
// includes

#include "gl_dd.h"
#include "camera.h"
class MaterialData_wu2;

// ======================================================================
// defines

class GL_wu2 : public GL_dd
{
private:

	// disable copy constructor and assignment operator
	GL_wu2(const GL_wu2 &);
	GL_wu2 &operator =(const GL_wu2 &);

private:

	// wireframe material
	static MaterialData_wu2       *wirefill;
	static const MaterialData_wu2 *material;

	static bool                    in3dRender;

	static float                   vx0;
	static float                   vy0;
	static float                   vx1;
	static float                   vy1;

private:

	// submit transformed/clipped primitives to the rasterizer
	static void passPointList(const TVertex *vertexList, int numberOfVertices);
	static void passLineList(const TVertex *vertexList, int numberOfVertices);
	static void passLineStrip(const TVertex *vertexList, int numberOfVertices);
	static void passTriangleList(const TVertex *vertexList, int numberOfVertices);
	static void passTriangleStrip(const TVertex *vertexList, int numberOfVertices);
	static void passTriangleFan(const TVertex *vertexList, int numberOfVertices);

public:

	static void  makeClut(const RGB *rgb, int numColors, byte **mem, void **clut);
	static int   getShadeBits(void);
	
public:

	GL_wu2(void);
	virtual ~GL_wu2(void);

	// version information
	virtual void  getInformation(Information *information);

	// see if the is usable
	virtual bool  query(int minor);

	// initialize
	virtual bool init(const GL_input *input, GL_output *output);

	// dump debug informtion
	virtual void  debugDump(void);

	// application activation
	virtual void  setActive(bool active);

	// access functions
	virtual bool lock(void);
	virtual void unlock(bool display);

	// set processing of primitives
	virtual void setPrimitiveProcessing(PrimitiveProcessing primitiveProcessing);

	// palette functions
	virtual PaletteData  *paletteCreateData(const Palette *pal);
	virtual void          paletteDestroyData(PaletteData *palData);

	// material functions
	virtual MaterialData *materialCreateData(const StaticMaterial *material);
	virtual void          materialDestroyData(MaterialData *mat);

	// material functions
	virtual void          setMaterial(const StaticMaterial *material);

	// 3d transformation setup
	virtual void beginRender(const Camera &camera);
	virtual void endRender(void);

	// 3d render setup
	virtual void setLightingEnabled(bool lightingEnabled);
	virtual void setLightList(Light **lightList, int lightCount);
	virtual bool setVertexList(const Vector *vertex, int vertexCount, const Vector *normal, int normalCount);
	virtual void setObjectToWorld(const Xform &objectToWorld);

	// object-space primitives
	virtual void drawPointList(const UVertex *vertexList, int numberOfVertices);
	virtual void drawLineList(const UVertex *vertexList, int numberOfVertices);
	virtual void drawLineStrip(const UVertex *vertexList, int numberOfVertices);
	virtual void drawTriangleList(const UVertex *vertexList, int numberOfVertices);
	virtual void drawTriangleStrip(const UVertex *vertexList, int numberOfVertices);
	virtual void drawTriangleFan(const UVertex *vertexList, int numberOfVertices);

	// screen-space primitives
	virtual void drawPointList(const TVertex *vertexList, int numberOfVertices);
	virtual void drawLineList(const TVertex *vertexList, int numberOfVertices);
	virtual void drawLineStrip(const TVertex *vertexList, int numberOfVertices);
	virtual void drawTriangleList(const TVertex *vertexList, int numberOfVertices);
	virtual void drawTriangleStrip(const TVertex *vertexList, int numberOfVertices);
	virtual void drawTriangleFan(const TVertex *vertexList, int numberOfVertices);
};

// ======================================================================

inline int GL_wu2::getShadeBits(void)
{
	return 6;
}

// ======================================================================

#endif

